@@ -6,7 +6,11 @@ import stat
 import tempfile
 import requests
 import time
+import sys
+import json
+import base64
 from .constants import *
+from packaging import version
 from knack.log import get_logger
 from knack.prompting import prompt_y_n
 from knack.prompting import NoTTYException
@@ -344,3 +348,66 @@ def graph_client_factory(cli_ctx, **_):
                                        base_url=cli_ctx.cloud.endpoints.active_directory_graph_resource_id)
     configure_common_settings(cli_ctx, client)
     return client
+
+
+def fetch_pop_publickey_kid(api_server_port, clientproxy_process):
+    requestbody = {}
+    poppublickey_uri = f'https://localhost:{api_server_port}/identity/poppublickey'
+    # Needed to prevent skip tls warning from printing to the console
+    original_stderr = sys.stderr
+    f = open(os.devnull, 'w')
+    sys.stderr = f
+
+    get_publickey_response = make_api_call_with_retries(poppublickey_uri, requestbody, "get", False, Get_PublicKey_Info_Fault_Type,
+                                                        'Failed to fetch public key info from clientproxy',
+                                                        "Failed to fetch public key info from client proxy", clientproxy_process)
+
+    sys.stderr = original_stderr
+    publickey_info = json.loads(get_publickey_response.text)
+    kid = publickey_info['publicKey']['kid']
+
+    return kid
+
+
+def fetch_and_post_at_to_csp(cmd, api_server_port, tenantId, kid, clientproxy_process):
+    req_cnfJSON = {"kid": kid, "xms_ksl": "sw"}
+    req_cnf = base64.urlsafe_b64encode(json.dumps(req_cnfJSON).encode('utf-8')).decode('utf-8')
+
+    # remove padding '=' character
+    if req_cnf[len(req_cnf) - 1] == '=':
+        req_cnf = req_cnf[:-1]
+
+    token_data = {"token_type": "pop", "key_id": kid, "req_cnf": req_cnf}
+    profile = Profile(cli_ctx=cmd.cli_ctx)
+    try:
+        credential, _, _ = profile.get_login_credentials(subscription_id=profile.get_subscription()["id"], resource=KAP_1P_Server_App_Scope)
+        accessToken = credential.get_token(KAP_1P_Server_App_Scope, data=token_data)
+        jwtToken = accessToken.token
+    except Exception as e:
+        telemetry.set_exception(exception=e, fault_type=Post_AT_To_ClientProxy_Failed_Fault_Type,
+                                summary='Failed to fetch access token using the PoP public key sent by client proxy')
+        close_subprocess_and_raise_cli_error(clientproxy_process, 'Failed to post access token to client proxy' + str(e))
+
+    jwtTokenData = {"accessToken": jwtToken, "serverId": KAP_1P_Server_AppId, "tenantID": tenantId, "kid": kid}
+    post_at_uri = f'https://localhost:{api_server_port}/identity/at'
+    # Needed to prevent skip tls warning from printing to the console
+    original_stderr = sys.stderr
+    f = open(os.devnull, 'w')
+    sys.stderr = f
+    post_at_response = make_api_call_with_retries(post_at_uri, jwtTokenData, "post", False, PublicKey_Export_Fault_Type, 'Failed to post access token to client proxy', "Failed to post access token to client proxy", clientproxy_process)
+
+    sys.stderr = original_stderr
+    return post_at_response
+
+
+def is_cli_using_msal_auth():
+        response_cli_version = az_cli("version --output json")
+        try:
+            cli_version = response_cli_version['azure-cli']
+        except Exception as ex:
+            raise CLIInternalError("Unable to decode the az cli version installed: {}".format(str(ex)))
+        if version.parse(cli_version) >= version.parse(AZ_CLI_ADAL_TO_MSAL_MIGRATE_VERSION):
+            return True
+        else:
+            return False
+
