@@ -45,6 +45,7 @@ def client_side_proxy_wrapper(cmd,
                               context_name=None):
 
     cloud = send_cloud_telemetry(cmd, cli_extension_name)
+    tenantId = graph_client_factory(cmd.cli_ctx).config.tenant_id
     if int(client_proxy_port) == int(api_server_port):
         raise ClientRequestError(f'Proxy uses port {api_server_port} internally.',
                                  recommendation='Please pass some other unused port through --port option.')
@@ -165,7 +166,6 @@ def client_side_proxy_wrapper(cmd,
         account = get_subscription_id(cmd.cli_ctx)
         account = Profile().get_subscription(account)
         user_type = account['user']['type']
-        tenantId = graph_client_factory(cmd.cli_ctx).config.tenant_id
 
         if user_type == 'user':
             dict_file = {'server': {'httpPort': int(client_proxy_port), 'httpsPort': int(
@@ -177,42 +177,42 @@ def client_side_proxy_wrapper(cmd,
         if cloud == 'DOGFOOD':
             dict_file['cloud'] = 'AzureDogFood'
 
-        # Fetching creds
-        creds_location = os.path.expanduser(os.path.join('~', creds_string))
-        try:
-            with open(creds_location) as f:
-                creds_list = json.load(f)
-        except Exception as e:
-            telemetry.set_exception(exception=e, fault_type=Load_Creds_Fault_Type,
-                                    summary='ArcProxyUtilities:Unable to load accessToken.json')
-            raise FileOperationError("Failed to load credentials." + str(e))
+        if not is_cli_using_msal_auth():
+            # Fetching creds
+            creds_location = os.path.expanduser(os.path.join('~', creds_string))
+            try:
+                with open(creds_location) as f:
+                    creds_list = json.load(f)
+            except Exception as e:
+                telemetry.set_exception(exception=e, fault_type=Load_Creds_Fault_Type,
+                                        summary='ArcProxyUtilities:Unable to load accessToken.json')
+                raise FileOperationError("Failed to load credentials." + str(e))
 
-        user_name = account['user']['name']
+            user_name = account['user']['name']
 
-        if user_type == 'user':
-            key = 'userId'
-            key2 = 'refreshToken'
-        else:
-            key = 'servicePrincipalId'
-            key2 = 'accessToken'
+            if user_type == 'user':
+                key = 'userId'
+                key2 = 'refreshToken'
+            else:
+                key = 'servicePrincipalId'
+                key2 = 'accessToken'
 
-        for i in range(len(creds_list)):
-            creds_obj = creds_list[i]
+            for i in range(len(creds_list)):
+                creds_obj = creds_list[i]
 
-            if key in creds_obj and creds_obj[key] == user_name:
-                creds = creds_obj[key2]
-                break
+                if key in creds_obj and creds_obj[key] == user_name:
+                    creds = creds_obj[key2]
+                    break
 
-        if creds == '':
-            telemetry.set_exception(exception='Credentials of user not found.', fault_type=Creds_NotFound_Fault_Type,
-                                    summary='ArcProxyUtilities:Unable to find creds of user')
-            raise UnclassifiedUserFault("Credentials of user not found.")
+            if creds == '':
+                telemetry.set_exception(exception='Credentials of user not found.', fault_type=Creds_NotFound_Fault_Type,
+                                        summary='ArcProxyUtilities:Unable to find creds of user')
+                raise UnclassifiedUserFault("Credentials of user not found.")
 
-        if user_type != 'user':
-            dict_file['identity']['clientSecret'] = creds
+            if user_type != 'user':
+                dict_file['identity']['clientSecret'] = creds
     else:
-        dict_file = {'server': {'httpPort': int(
-            client_proxy_port), 'httpsPort': int(api_server_port)}}
+        dict_file = {'server': {'httpPort': int(client_proxy_port), 'httpsPort': int(api_server_port)}}
 
     telemetry.set_debug_info('User type is ', user_type)
 
@@ -231,11 +231,12 @@ def client_side_proxy_wrapper(cmd,
     if '--debug' in cmd.cli_ctx.data['safe_params']:
         args.append("-d")
         debug_mode = True
-    client_side_proxy_main(cmd, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, args, client_proxy_port,
+    client_side_proxy_main(cmd, tenantId, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, args, client_proxy_port,
                            api_server_port, operating_system, creds, user_type, cloud, debug_mode, token=token, path=path, context_name=context_name, clientproxy_process=None)
 
 
 def client_side_proxy_main(cmd,
+                           tenantId,
                            client,
                            resource_group_name,
                            resource_provider_name,
@@ -254,7 +255,7 @@ def client_side_proxy_main(cmd,
                                '~'), '.kube', 'config'),
                            context_name=None,
                            clientproxy_process=None):
-    expiry, clientproxy_process = client_side_proxy(cmd, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, 0, args, client_proxy_port,
+    expiry, clientproxy_process = client_side_proxy(cmd, tenantId, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, 0, args, client_proxy_port,
                                                     api_server_port, operating_system, creds, user_type, cloud, debug_mode, token=token, path=path, context_name=context_name, clientproxy_process=None)
     next_refresh_time = expiry - CSP_REFRESH_TIME
 
@@ -262,7 +263,7 @@ def client_side_proxy_main(cmd,
         time.sleep(60)
         if(is_csp_running(clientproxy_process)):
             if time.time() >= next_refresh_time:
-                expiry, clientproxy_process = client_side_proxy(cmd, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, 1, args, client_proxy_port,
+                expiry, clientproxy_process = client_side_proxy(cmd, tenantId, client, resource_group_name, resource_provider_name, resource_type_name, cluster_name, 1, args, client_proxy_port,
                                                                 api_server_port, operating_system, creds, user_type, cloud, debug_mode, token=token, path=path, context_name=context_name, clientproxy_process=clientproxy_process)
                 next_refresh_time = expiry - CSP_REFRESH_TIME
         else:
@@ -272,6 +273,7 @@ def client_side_proxy_main(cmd,
 
 
 def client_side_proxy(cmd,
+                      tenantId,
                       client,
                       resource_group_name,
                       resource_provider_name,
@@ -325,20 +327,38 @@ def client_side_proxy(cmd,
                                     summary='ArcProxyUtilities:Unable to run client proxy executable')
             raise CLIInternalError("Failed to start proxy process." + str(e))
 
-        if user_type == 'user':
-            identity_data = {}
-            identity_data['refreshToken'] = creds
-            identity_uri = f'https://localhost:{api_server_port}/identity/rt'
+        if not is_cli_using_msal_auth(): # refresh token approach if cli is using ADAL auth. This is for cli < 2.30.0
+            if user_type == 'user':
+                identity_data = {}
+                identity_data['refreshToken'] = creds
+                identity_uri = f'https://localhost:{api_server_port}/identity/rt'
 
-            # To prevent skip tls warning from printing to the console
-            original_stderr = sys.stderr
-            f = open(os.devnull, 'w')
-            sys.stderr = f
+                # To prevent skip tls warning from printing to the console
+                original_stderr = sys.stderr
+                f = open(os.devnull, 'w')
+                sys.stderr = f
 
-            make_api_call_with_retries(identity_uri, identity_data, "post", False, Post_RefreshToken_Fault_Type,
+                make_api_call_with_retries(identity_uri, identity_data, "post", False, Post_RefreshToken_Fault_Type,
                                              'Unable to post refresh token details to clientproxy',
                                              "Failed to pass refresh token details to proxy.", clientproxy_process)
-            sys.stderr = original_stderr
+                sys.stderr = original_stderr
+    
+    
+    if token is None:
+        if is_cli_using_msal_auth():  # jwt token approach if cli is using MSAL. This is for cli >= 2.30.0
+            kid = fetch_pop_publickey_kid(api_server_port, clientproxy_process)
+            post_at_response = fetch_and_post_at_to_csp(cmd, api_server_port, tenantId, kid, clientproxy_process)
+
+            if post_at_response.status_code != 200:
+                if post_at_response.status_code == 500 and "public key expired" in post_at_response.text:  # pop public key must have been rotated
+                    telemetry.set_exception(exception=post_at_response.text, fault_type=PoP_Public_Key_Expried_Fault_Type,
+                                            summary='PoP public key has expired')
+                    kid = fetch_pop_publickey_kid(api_server_port, clientproxy_process)  # fetch the rotated PoP public key
+                    fetch_and_post_at_to_csp(cmd, api_server_port, tenantId, kid, clientproxy_process)  # fetch and post the at corresponding to the new public key
+                else:
+                    telemetry.set_exception(exception=post_at_response.text, fault_type=Post_AT_To_ClientProxy_Failed_Fault_Type,
+                                            summary='Failed to post access token to client proxy')
+                    close_subprocess_and_raise_cli_error(clientproxy_process, 'Failed to post access token to client proxy' + post_at_response.text)
 
     if token is not None:
         data = prepare_data_for_csp_token_case(
